/*
 * Copyright (c) 2010, Stephen Denne
 */
package nz.co.datacute.pgquilt;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;

import nz.co.datacute.treemap.Node;
import nz.co.datacute.treemap.NodeCategory;

import android.content.Context;

public class DemoCreator {
	public static Node getRoot(Context context) throws FileNotFoundException,
			IOException {
		
		InputStream input = context.getResources().getAssets().open("sample.csv");
		//TODO use strings.xml
		Node root = new Node("Database Tables", "Database Tables", new String[] {"Disk Space"}, new String[] {"Type"});
		
		Map<String,Node> schemas = new HashMap<String,Node>();
		
		NodeCategory row = new NodeCategory("rows", 0xFF0097C8);
		NodeCategory index = new NodeCategory("indexes", 0xFFFFFF00);
		NodeCategory toast = new NodeCategory("toast", 0xFFFF0000);
		
		BufferedReader br = new BufferedReader(new InputStreamReader(input));
		String line;
		while ((line = br.readLine()) != null) {
			LineParser rs = new LineParser(line);
			double rowsWeight = rs.getDouble(30);
			double toastWeight = rs.getDouble(32);
			double indexesWeight = rs.getDouble(34);
			if ((rowsWeight + toastWeight + indexesWeight) > 0.0) {
				String schema = rs.getString(1);
				Node schemaNode = schemas.get(schema);
				if (schemaNode == null) {
					schemaNode = new Node(schema,schema);
					schemas.put(schema, schemaNode);
				}
				String tableName = rs.getString(2);
				Node table = new Node(schema + "." + tableName, tableName);
				if (rowsWeight > 0.0) {
					//TODO use strings.xml
					table.add(new Node(schema + "." + tableName + " rows", tableName, new double[] {rowsWeight}, new String[] {rs.getString(31)}, new NodeCategory[] {row}));
				}
				if (toastWeight > 0.0) {
					//TODO use strings.xml
					table.add(new Node(schema + "." + tableName + " toast", tableName, new double[] {toastWeight}, new String[] {rs.getString(33)}, new NodeCategory[] {toast}));
				}
				if (indexesWeight > 0.0) {
					//TODO use strings.xml
					table.add(new Node(schema + "." + tableName + " indexes", tableName, new double[] {indexesWeight}, new String[] {rs.getString(35)}, new NodeCategory[] {index}));
				}
				schemaNode.add(table);
			}
		}
		if (schemas.size() == 1) {
			return schemas.values().iterator().next();
		}
		for (Node schemaNode : schemas.values()) {
			root.add(schemaNode);
		}
		return root;
	}

	static private class LineParser {
		private final String[] fields;
		public LineParser(String line) {
			fields = line.split(",");
		}
		public String getString(int i) {
			String result = fields[i-1];
			if (result.startsWith("\"") &&  result.endsWith("\"")) {
				return result.substring(1, result.length()-1);
			}
			return result;
		}
		public double getDouble(int i) {
			return Double.valueOf(fields[i-1]);
		}
	}
}
