/*
 * Copyright (c) 2010, Stephen Denne
 */
package nz.co.datacute.pgquilt;

import java.util.List;

import nz.co.datacute.treemap.Node;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.OnSharedPreferenceChangeListener;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;

public class QuiltViewer extends Activity implements OnSharedPreferenceChangeListener {

	private TreeMapView treeMap;
	private boolean needRefresh = false;

	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(getBaseContext());
        prefs.registerOnSharedPreferenceChangeListener(this);
        setContentView(R.layout.main);
		treeMap = (TreeMapView) findViewById(R.id.tree_map);
		treeMap.setState(getLastNonConfigurationInstance());
//        findViewById(R.id.filter).setOnClickListener(new View.OnClickListener() {
//			@Override
//			public void onClick(View v) {
//		    	// TODO treeMap.filter()
//			}
//        });
        findViewById(R.id.weighting_selection).setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
		        treeMap.changeWeighting((treeMap.getRoot().getChosenWeight() + 1) % treeMap.getRoot().getAvailableWeightings());
			}
        });
        findViewById(R.id.category_change).setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
		        treeMap.changeCategory((treeMap.getRoot().getChosenCategory() + 1) % treeMap.getRoot().getAvailableCategories());
			}
        });
        final View refreshButton = findViewById(R.id.refresh);
        refreshButton.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				refreshButton.setEnabled(false);
				treeMap.refresh();
			}
        });
    }

    @Override
    public Object onRetainNonConfigurationInstance() {
        return treeMap.getState();
    }
    
    @Override
    protected void onRestart() {
    	super.onRestart();
		if (needRefresh) {
			needRefresh = false;
			treeMap.refresh();
		}
    }
    
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.options_menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
//        case R.id.menu_filter:
//			AlertDialog.Builder filterDialog = new AlertDialog.Builder(QuiltViewer.this);
//			filterDialog.setTitle(R.string.dialog_title_filter);
//			Node root = treeMap.getRoot();
//			List<Node> children = root.getChildren();
//			String[] schemas = new String[children.size()];
//			boolean[] selection = new boolean[schemas.length];
//			for (int i=0; i < schemas.length; i++) {
//				schemas[i] = children.get(i).shortLabel;
//				selection[i] = true;
//			}
//			filterDialog.setMultiChoiceItems(schemas, selection, new DialogInterface.OnMultiChoiceClickListener() {
//				public void onClick(DialogInterface dialog, int which, boolean isChecked) {
//			    	// TODO treeMap.filter()
//				}
//			});
//			filterDialog.show();
//            return true;
        case R.id.menu_weightings:
			AlertDialog.Builder weightingsDialog = new AlertDialog.Builder(QuiltViewer.this);
			weightingsDialog.setTitle(R.string.dialog_title_weighting);
			weightingsDialog.setSingleChoiceItems(treeMap.getRoot().weightLabels, treeMap.getRoot().getChosenWeight(), new DialogInterface.OnClickListener() {
			    public void onClick(DialogInterface dialog, int item) {
			        treeMap.changeWeighting(item);
			    }
			});
			weightingsDialog.show();
            return true;
        case R.id.menu_categories:
			AlertDialog.Builder categoriesDialog = new AlertDialog.Builder(QuiltViewer.this);
			categoriesDialog.setTitle(R.string.dialog_title_category);
			categoriesDialog.setSingleChoiceItems(treeMap.getRoot().categoryLabels, treeMap.getRoot().getChosenCategory(), new DialogInterface.OnClickListener() {
			    public void onClick(DialogInterface dialog, int item) {
			        treeMap.changeCategory(item);
			    }
			});
			categoriesDialog.show();
            return true;
        case R.id.about:
			new AlertDialog.Builder(this)
				.setTitle(R.string.about_title)
				.setMessage(R.string.about)
				.setCancelable(true)
				.show();
            return true;
        case R.id.refresh:
    		treeMap.refresh();
            return true;
        case R.id.preferences:
        	startActivity(new Intent(getBaseContext(), ConnectionPreferences.class));
            return true;
        default:
            return false;
        }
    }
    
    @Override
    public void onLowMemory() {
    	super.onLowMemory();
    	if (treeMap != null) treeMap.onLowMemory();
    }

	@Override
	public void onSharedPreferenceChanged(SharedPreferences sharedPreferences, String key) {
		needRefresh = true;
	}
}