/*
 * Copyright (c) 2010, Stephen Denne
 */
package nz.co.datacute.pgquilt;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Map;

import nz.co.datacute.pgquilt.R;
import nz.co.datacute.treemap.Node;
import nz.co.datacute.treemap.NodeCategory;
import nz.co.datacute.treemap.TreeMapException;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.res.Resources;
import android.content.res.TypedArray;
import android.preference.PreferenceManager;

public class SQLCreator {

	public static final String QUERY = 
		// Headings
		"select null::text as parent_oid,null::text as oid,'Database Tables'::text as short_label,'Database Tables'::text as full_label,'Table'::text as category_1,'Type'::text as category_2,'Tablespace'::text as category_3,'Owner'::text as category_4,null::bigint as weight_1, 'Disk Space'::text as weight_desc_1,null::bigint as weight_2, 'Tuples'::text as weight_desc_2 " +
		"union all " + // Schemas
		"select null,oid::text,nspname,nspname,null,null,null,null,null,null,null,null from pg_namespace " +
		"union all " + // Tables
		"select relnamespace::text,cl.oid::text,relname,nspname||'.'||relname,relname,null,null,rolname,null,null,null,null from pg_class cl inner join pg_namespace n on n.oid=relnamespace inner join pg_roles r on r.oid=cl.relowner where relkind='r' " +
		"union all " + // Table Rows
		"select cl.oid::text,cl.oid::text||'_rows',relname,nspname||'.'||relname||' rows',null,'rows',coalesce(spcname,'pg_default'),null,pg_relation_size(cl.oid),pg_size_pretty(pg_relation_size(cl.oid)),reltuples::bigint,reltuples::bigint::text from pg_class cl inner join pg_namespace n on n.oid=relnamespace left join pg_tablespace t on t.oid=cl.reltablespace where relkind='r' " +
		"union all " + // Table Indexes
		"select cl.oid::text,cl.oid::text||'_indexes',relname||' indexes',nspname||'.'||relname||' indexes',null,'index',null,null,null,null,null,null from pg_class cl inner join pg_namespace n on n.oid=relnamespace where relkind='r' and cl.oid in (select indrelid from pg_index) " +
		"union all " + // Indexes
		"select indrelid||'_indexes',indexrelid::text,relname,nspname||'.'||relname,null,null,coalesce(spcname,'pg_default'),null,pg_relation_size(indexrelid),pg_size_pretty(pg_relation_size(indexrelid)),reltuples::bigint,reltuples::bigint::text from pg_class cl inner join pg_namespace n on n.oid=relnamespace inner join pg_index on indexrelid=cl.oid left join pg_tablespace t on t.oid=cl.reltablespace where relkind='i' and indrelid not in (select reltoastrelid from pg_class)" +
		"union all " + // Table Toast
		"select cl.oid::text,cl.oid::text||'_toast',relname,nspname||'.'||relname||' toast',null,'toast',null,null,null,null,null,null from pg_class cl inner join pg_namespace n on n.oid=relnamespace where relkind='r' and reltoastrelid<>0 " +
		"union all " + // Table Toast Data
		"select cl.oid::text||'_toast',cl.oid::text||' toast data',relname,nspname||'.'||relname||' toast data',null,null,coalesce(spcname,'pg_default'),null,pg_relation_size(reltoastrelid),pg_size_pretty(pg_relation_size(reltoastrelid)),null,null from pg_class cl inner join pg_namespace n on n.oid=relnamespace left join pg_tablespace t on t.oid=cl.reltablespace where relkind='r' and reltoastrelid<>0 " +
		"union all " + // Table Toast Indexes
		"select cl.oid::text||'_toast',cl.oid::text||' toast index',cl.relname,nspname||'.'||cl.relname||' toast index',null,null,coalesce(spcname,'pg_default'),null,pg_relation_size(indexrelid),pg_size_pretty(pg_relation_size(indexrelid)),null,null from pg_class cl inner join pg_index on indrelid=reltoastrelid inner join pg_class cli on cli.oid=indexrelid inner join pg_namespace n on n.oid=cli.relnamespace left join pg_tablespace t on t.oid=cli.reltablespace where cl.relkind='r' and cl.reltoastrelid<>0 " +
		"";

	public static Node getRoot(Context context) throws ClassNotFoundException, SQLException {
		
		Map<String,Node> nodes = new HashMap<String,Node>();

		Resources resources = context.getResources();
		Map<String,NodeCategory> categories1 = new HashMap<String, NodeCategory>();
		Map<String,NodeCategory> categories2 = new HashMap<String, NodeCategory>();
		Map<String,NodeCategory> categories3 = new HashMap<String, NodeCategory>();
		Map<String,NodeCategory> categories4 = new HashMap<String, NodeCategory>();
		TypedArray colors = resources.obtainTypedArray(R.array.category_colour);

		Node root = null;

		Class.forName("org.postgresql.Driver");
		String url = getURL(context);
		SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(context);
		DriverManager.setLoginTimeout(Integer.valueOf(prefs.getString("connection_timeout","5")));
		Connection conn = DriverManager.getConnection(url);
		Statement st = conn.createStatement();
		ResultSet rs = st.executeQuery(QUERY);
		while (rs.next()) {
			String parent_oid = rs.getString(1);
			String oid = rs.getString(2);
			String shortName = rs.getString(3);
			String fullName = rs.getString(4);
			String categoryValue1 = rs.getString(5);
			String categoryValue2 = rs.getString(6);
			String categoryValue3 = rs.getString(7);
			String categoryValue4 = rs.getString(8);
			double weight1 = rs.getDouble(9);
			String weight1Desc = rs.getString(10);
			double weight2 = rs.getDouble(11);
			String weight2Desc = rs.getString(12);
			if ((oid == null) || (oid.length() == 0)) {
				// Read headings
				root = new Node(fullName, shortName, new String[] {weight1Desc, weight2Desc}, new String[] {categoryValue1,categoryValue2,categoryValue3,categoryValue4});
			} else {
				NodeCategory category1 = null;
				if (categoryValue1 != null) {
					category1 = categories1.get(categoryValue1);
					if (category1 == null) {
						category1 = new NodeCategory(categoryValue1,colors.getColor(categories1.size() % colors.length(),0xFF0097C8));
						categories1.put(categoryValue1, category1);
					}
				}
				NodeCategory category2 = null;
				if (categoryValue2 != null) {
					category2 = categories2.get(categoryValue2);
					if (category2 == null) {
						category2 = new NodeCategory(categoryValue2,colors.getColor(categories2.size() % colors.length(),0xFF0097C8));
						categories2.put(categoryValue2, category2);
					}
				}
				NodeCategory category3 = null;
				if (categoryValue3 != null) {
					category3 = categories3.get(categoryValue3);
					if (category3 == null) {
						category3 = new NodeCategory(categoryValue3,colors.getColor(categories3.size() % colors.length(),0xFF0097C8));
						categories3.put(categoryValue3, category3);
					}
				}
				NodeCategory category4 = null;
				if (categoryValue4 != null) {
					category4 = categories4.get(categoryValue4);
					if (category4 == null) {
						category4 = new NodeCategory(categoryValue4,colors.getColor(categories4.size() % colors.length(),0xFF0097C8));
						categories4.put(categoryValue4, category4);
					}
				}
				Node node = new Node(parent_oid, oid, fullName, shortName, new double[] {weight1, weight2}, new String[] {weight1Desc, weight2Desc}, new NodeCategory[] {category1, category2, category3, category4});
				nodes.put(oid, node);
			}
		}
		rs.close();
		st.close();
		conn.close();

		for (Node node : nodes.values()) {
			Node parent = nodes.get(node.parent_oid);
			if (parent != null) {
				parent.add(node);
			} else {
				root.add(node);
			}
		}

		return root;
	}

	private static String getURL(Context context) {
		Resources resources = context.getResources();

		SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(context);
		String host = prefs.getString("host", "");
		if ((host == null) || (host.length() == 0)) {
			throw new TreeMapException(resources.getString(R.string.ermsg_no_host));
		}
		String database = prefs.getString("db", "");
		if ((database == null) || (database.length() == 0)) {
			throw new TreeMapException(resources.getString(R.string.ermsg_no_database));
		}
		StringBuilder urlSB = new StringBuilder();
		urlSB.append("jdbc:postgresql://");
		urlSB.append(host);
		urlSB.append("/");
		urlSB.append(database);
		boolean firstAppend = appendOption("user", prefs.getString("user", ""), urlSB, true);
		firstAppend = appendOption("password", prefs.getString("password", ""), urlSB, firstAppend);
		firstAppend = appendOption(null, prefs.getString("other_connection_options", ""), urlSB, firstAppend);
		String url = urlSB.toString();
		return url;
	}

	private static boolean appendOption(String optionName, String optionValue, StringBuilder urlSB,
			boolean firstAppend) {
		if ((optionValue != null) && (optionValue.length() > 0)) {
			if (firstAppend) {
				firstAppend = false;
				urlSB.append("?");
			} else {
				urlSB.append("&");
			}
			if ((optionName != null) && (optionName.length() > 0)) {
				urlSB.append(optionName).append("=");
			}
			urlSB.append(optionValue);
		}
		return firstAppend;
	}
}
