/*
 * Copyright (c) 2010, Stephen Denne
 */
package nz.co.datacute.treemap;

import java.util.Arrays;

public class NodePositioner {
	
	public static void layout(final Node node, final int width, final int height) {
		node.width = width;
		node.height = height;
		if ((width == 0) || (height == 0)) return;
		if (!node.children.isEmpty()) {
			Node[] children = node.children.toArray(new Node[node.children.size()]);
			Arrays.sort(children);
			double remainingWeight = 0.0;
			for (Node child : children) {
				remainingWeight += child.getWeight();
			}
			final double sizeRatio = (width * height) / remainingWeight;
			double accumWeight = children[0].getWeight();
			int remWidth = width;
			int remHeight = height;
			int firstNode = 0;
			int lastNode;
			boolean stackingVertically = (remWidth > remHeight);
			int fixedDimension = stackingVertically ? remHeight : remWidth;
			double currentRatio = 0.0;
			boolean calculateCurrentRatio = true;
			for (lastNode = 1; lastNode < children.length; lastNode++) {
				final double firstChildWeight = children[firstNode].getWeight();
				final double lastChildWeight = children[lastNode].getWeight();
				if (calculateCurrentRatio) {
					final double fixedPortion = fixedDimension / accumWeight;
					currentRatio = Math.max(Math.floor(firstChildWeight * fixedPortion) / Math.floor(sizeRatio / fixedPortion), Math.floor(sizeRatio / fixedPortion) / Math.floor(children[lastNode-1].getWeight() * fixedPortion));
				}
				final double fixedPortion = fixedDimension / (accumWeight + lastChildWeight);
				final double newRatio = Math.max(Math.floor(firstChildWeight * fixedPortion) / Math.floor(sizeRatio / fixedPortion), Math.floor(sizeRatio / fixedPortion) / Math.floor(lastChildWeight * fixedPortion));
				if(currentRatio<newRatio) {
					final int rankSize = (int) Math.floor(((remWidth * remHeight) / remainingWeight) / (fixedDimension / accumWeight));
					positionChildren(node, children, firstNode, lastNode, remWidth, remHeight, accumWeight, rankSize, stackingVertically);
					remWidth -= stackingVertically ? rankSize : 0;
					remHeight -= stackingVertically ? 0 : rankSize;
					remainingWeight -= accumWeight;
					accumWeight = lastChildWeight;
					firstNode = lastNode;
					stackingVertically = (remWidth > remHeight);
					fixedDimension = stackingVertically ? remHeight : remWidth;
					calculateCurrentRatio = true;
				} else {
					currentRatio = newRatio;
					calculateCurrentRatio = false;
					accumWeight +=  lastChildWeight;
				}
			}
			int rankSize = (int) Math.floor(((remWidth * remHeight) / remainingWeight) / (fixedDimension / accumWeight));
			positionChildren(node, children, firstNode, lastNode, remWidth, remHeight, accumWeight, rankSize, stackingVertically);
			remWidth -= stackingVertically ? rankSize : 0;
			remHeight -= stackingVertically ? 0 : rankSize;
		}
	}

	private static void positionChildren(Node node, Node[] children, int firstNode, int lastNode, int remWidth, int remHeight, double accumWeight, int rankSize, boolean stackingVertically) {
		int accumPortion = 0;
		double accumWeightPortion = 0.0;
		for (int c = firstNode; c < lastNode; c++) {
			final Node child = children[c];
			child.x = node.x + (node.width - remWidth + (stackingVertically ? 0 : accumPortion));
			child.y = node.y + (node.height - remHeight + (stackingVertically ? accumPortion : 0));
			final int childPortion = ((c==lastNode-1)?(stackingVertically ? remHeight : remWidth):(int) Math.floor((accumWeightPortion + child.getWeight()) / accumWeight * (stackingVertically ? remHeight : remWidth))) - accumPortion;
			accumPortion += childPortion;
			accumWeightPortion += child.getWeight();
			NodePositioner.layout(child, (stackingVertically ? rankSize : childPortion), (stackingVertically ? childPortion : rankSize));
		}
	}
}
